/*
  IR Receiver

  T. Nakagawa

  2011-07-19
*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "uart.h"


#define OSCCALVAL 164
#define BUF_SIZE 256

volatile static int Buf_wp = 0;
volatile static int Buf_rp = 0;
volatile static int Buf_num = 0;
volatile static unsigned int Buf[BUF_SIZE];


static void buf_put(unsigned int data) {
  if (Buf_num < BUF_SIZE) {
    Buf[Buf_wp++] = data;
    Buf_num++;
    if (Buf_wp == BUF_SIZE) Buf_wp = 0;
  }
}


static unsigned int buf_get(void) {
  unsigned int data = 0;

  if (Buf_num > 0) {
    data = Buf[Buf_rp++];
    Buf_num--;
    if (Buf_rp == BUF_SIZE) Buf_rp = 0;
  }

  return data;
}


static void send_hex(unsigned char val) {
  if (val < 10) val += '0'; else val += 'a' - 10;
  uart_putchar(val);

  return;
}


static void send_data(void) {
  unsigned int data;

  data = buf_get();
  if (data == 0) {
    uart_putchar('\r');
    uart_putchar('\n');
  } else {
    send_hex(data >> 12);
    send_hex((unsigned char)(data >> 8) & 0x0f);
    send_hex((unsigned char)(data >> 4) & 0x0f);
    send_hex((unsigned char)data & 0x0f);
    uart_putchar(' ');
  }

  return;
}


int main(void) {
  /* NbN̏C(8,294,400Hz) */
  OSCCAL = OSCCALVAL;

  /* |[g(PB0: IR sensor input) */
  PORTB = ~_BV(PB0);
  PORTC = 0xff;
  PORTD = 0xff;

  /* UART */
  uart_init();

  /* AiORp[^~ */
  ACSR |= _BV(ACD);

  /* X[v */
  set_sleep_mode(SLEEP_MODE_IDLE);

  /* ^C}EJE^ */
  TCCR1B |= (_BV(ICES1) | 0x03);
  TIMSK1 |= (_BV(ICIE1) | _BV(TOIE1));
  sei();

  for (; ; ) {
    cli();

    /* f[^Ώo */
    if (Buf_num > 0) {
      sei();
      send_data();
      continue;
    }

    /* 荞݂҂ */
    sleep_enable();
    sei();
    sleep_cpu();
    sleep_disable();
  }

  return 0;
}


/*
  ^C}1I[o[t[荞
*/
ISR (TIMER1_OVF_vect) {
  /* I[o[t[荞݂֎~ */
  TIMSK1 &= ~_BV(TOIE1);

  /* GbW̃Lv`ɐݒ */
  TCCR1B &= ~_BV(ICES1);

  /* s𑗐M */
  buf_put(0);
}


/*
  ^C}1Lv`荞
*/
ISR (TIMER1_CAPT_vect) {
  /* I[o[t[NĂȂ΃JE^l𑗐M */
  if (TIMSK1 & _BV(TOIE1)) {
    buf_put(ICR1);
  }

  /* 荞ݔ̃GbW𔽓] */
  TCCR1B ^= _BV(ICES1);

  /* JE^Zbg */
  TCNT1 = 0x0000;

  /* I[o[t[荞݂ėvtONA */
  TIMSK1 |= _BV(TOIE1);
  TIFR1 |= _BV(TOV1);
}
