; IR Remote Controller (Canon RC-1/RC-5)
; 2011-08-12
; T. Nakagawa

; CPU Clock: 1MHz (Internal RC Osc.)
; PD[0123456]: IR-LED

.include "tn2313def.inc"

.def	irled_off=r16
.def	irled_on=r17

; interrupt vector
	rjmp	reset
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	reti
	rjmp	pcint
	reti
	reti
	reti
	reti
	reti
	reti
	reti

reset:
	; stack
	ldi	ZL, RAMEND
	out	SPL, ZL
	; register
	ldi	irled_off, 0xff
	ldi	irled_on, 0x00
	; I/O
	ldi	ZL, 0xff
	out	PORTA, ZL
	out	PORTB, ZL
	out	PORTD, ZL
	out	DDRD, ZL
	; sleep mode (power-down)
	ldi	ZL, (EXP2(SE) | EXP2(SM0))
	out	MCUCR, ZL
	; analog comparator off
	ldi	ZL, EXP2(ACD)
	out	ACSR, ZL
	; enable interruption
	ldi	ZL, 0xff
	out	PCMSK, ZL
	ldi	ZL, EXP2(PCIE)
	out	GIMSK, ZL
	sei
main:
	sleep
	rjmp	main

pcint:
	sbis	PINB, 0
	rcall	sig_0
	sbis	PINB, 4
	rcall	sig_4
	reti

sig_0:
	rcall	cls_0
	rcall	cls_2
	ret

sig_4:
	rcall	cls_1
	rcall	cls_2
	ret

cls_0:
	ldi	ZL, 16
	rcall	pulse
	ldi	ZL, 236	; 7.33ms
	rcall	space
	ret

cls_1:
	ldi	ZL, 16
	rcall	pulse
	ldi	ZL, 173	; 5.36ms
	rcall	space
	ret

cls_2:
	ldi	ZL, 16
	rcall	pulse
	ldi	ZL, 255
	rcall	space
	ret

; Pulse (32kHz, 1/3 duty) (31*ZL+3 clocks)
pulse:	out	PORTD, irled_on
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	out	PORTD, irled_off
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	dec	ZL
	brne	pulse
	ret

; Space (T=1/32kHz) (31*ZL+3 clocks)
space:	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	dec	ZL
	brne	space
	ret
